/* libguestfs generated file
 * WARNING: THIS FILE IS GENERATED FROM THE FOLLOWING FILES:
 *          generator/erlang.ml
 *          and from the code in the generator/ subdirectory.
 * ANY CHANGES YOU MAKE TO THIS FILE WILL BE LOST.
 *
 * Copyright (C) 2009-2020 Red Hat Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef GUESTFS_ERLANG_ACTIONS_H_
#define GUESTFS_ERLANG_ACTIONS_H_

extern guestfs_h *g;

extern int dispatch (ei_x_buff *retbuff, const char *buff, int *index);
extern int make_error (ei_x_buff *retbuff, const char *funname);
extern int unknown_optarg (ei_x_buff *retbuff, const char *funname, const char *optargname);
extern int unknown_function (ei_x_buff *retbuff, const char *fun);
extern int make_string_list (ei_x_buff *buff, char **r);
extern int make_table (ei_x_buff *buff, char **r);
extern int make_bool (ei_x_buff *buff, int r);
extern int atom_equals (const char *atom, const char *name);
extern int decode_string_list (const char *buff, int *index, char ***res);
extern int decode_string (const char *buff, int *index, char **res);
extern int decode_binary (const char *buff, int *index, char **res, size_t *size);
extern int decode_bool (const char *buff, int *index, int *res);
extern int decode_int (const char *buff, int *index, int *res);
extern int decode_int64 (const char *buff, int *index, int64_t *res);

int make_application (ei_x_buff *buff, const struct guestfs_application *application);
int make_application2 (ei_x_buff *buff, const struct guestfs_application2 *application2);
int make_btrfsbalance (ei_x_buff *buff, const struct guestfs_btrfsbalance *btrfsbalance);
int make_btrfsqgroup (ei_x_buff *buff, const struct guestfs_btrfsqgroup *btrfsqgroup);
int make_btrfsscrub (ei_x_buff *buff, const struct guestfs_btrfsscrub *btrfsscrub);
int make_btrfssubvolume (ei_x_buff *buff, const struct guestfs_btrfssubvolume *btrfssubvolume);
int make_dirent (ei_x_buff *buff, const struct guestfs_dirent *dirent);
int make_hivex_node (ei_x_buff *buff, const struct guestfs_hivex_node *hivex_node);
int make_hivex_value (ei_x_buff *buff, const struct guestfs_hivex_value *hivex_value);
int make_inotify_event (ei_x_buff *buff, const struct guestfs_inotify_event *inotify_event);
int make_int_bool (ei_x_buff *buff, const struct guestfs_int_bool *int_bool);
int make_isoinfo (ei_x_buff *buff, const struct guestfs_isoinfo *isoinfo);
int make_lvm_lv (ei_x_buff *buff, const struct guestfs_lvm_lv *lvm_lv);
int make_lvm_pv (ei_x_buff *buff, const struct guestfs_lvm_pv *lvm_pv);
int make_lvm_vg (ei_x_buff *buff, const struct guestfs_lvm_vg *lvm_vg);
int make_mdstat (ei_x_buff *buff, const struct guestfs_mdstat *mdstat);
int make_partition (ei_x_buff *buff, const struct guestfs_partition *partition);
int make_stat (ei_x_buff *buff, const struct guestfs_stat *stat);
int make_statns (ei_x_buff *buff, const struct guestfs_statns *statns);
int make_statvfs (ei_x_buff *buff, const struct guestfs_statvfs *statvfs);
int make_tsk_dirent (ei_x_buff *buff, const struct guestfs_tsk_dirent *tsk_dirent);
int make_utsname (ei_x_buff *buff, const struct guestfs_utsname *utsname);
int make_version (ei_x_buff *buff, const struct guestfs_version *version);
int make_xattr (ei_x_buff *buff, const struct guestfs_xattr *xattr);
int make_xfsinfo (ei_x_buff *buff, const struct guestfs_xfsinfo *xfsinfo);
int make_yara_detection (ei_x_buff *buff, const struct guestfs_yara_detection *yara_detection);
int make_lvm_lv_list (ei_x_buff *buff, const struct guestfs_lvm_lv_list *lvm_lvs);
int make_dirent_list (ei_x_buff *buff, const struct guestfs_dirent_list *dirents);
int make_btrfsqgroup_list (ei_x_buff *buff, const struct guestfs_btrfsqgroup_list *btrfsqgroups);
int make_tsk_dirent_list (ei_x_buff *buff, const struct guestfs_tsk_dirent_list *tsk_dirents);
int make_partition_list (ei_x_buff *buff, const struct guestfs_partition_list *partitions);
int make_statns_list (ei_x_buff *buff, const struct guestfs_statns_list *statnss);
int make_application2_list (ei_x_buff *buff, const struct guestfs_application2_list *application2s);
int make_inotify_event_list (ei_x_buff *buff, const struct guestfs_inotify_event_list *inotify_events);
int make_application_list (ei_x_buff *buff, const struct guestfs_application_list *applications);
int make_hivex_value_list (ei_x_buff *buff, const struct guestfs_hivex_value_list *hivex_values);
int make_xattr_list (ei_x_buff *buff, const struct guestfs_xattr_list *xattrs);
int make_lvm_pv_list (ei_x_buff *buff, const struct guestfs_lvm_pv_list *lvm_pvs);
int make_yara_detection_list (ei_x_buff *buff, const struct guestfs_yara_detection_list *yara_detections);
int make_lvm_vg_list (ei_x_buff *buff, const struct guestfs_lvm_vg_list *lvm_vgs);
int make_btrfssubvolume_list (ei_x_buff *buff, const struct guestfs_btrfssubvolume_list *btrfssubvolumes);
int make_mdstat_list (ei_x_buff *buff, const struct guestfs_mdstat_list *mdstats);
int make_hivex_node_list (ei_x_buff *buff, const struct guestfs_hivex_node_list *hivex_nodes);
int make_stat_list (ei_x_buff *buff, const struct guestfs_stat_list *stats);

int run_acl_delete_def_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_acl_get_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_acl_set_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_add_cdrom (ei_x_buff *retbuff, const char *buff, int *index);
int run_add_domain (ei_x_buff *retbuff, const char *buff, int *index);
int run_add_drive (ei_x_buff *retbuff, const char *buff, int *index);
int run_add_drive_ro (ei_x_buff *retbuff, const char *buff, int *index);
int run_add_drive_ro_with_if (ei_x_buff *retbuff, const char *buff, int *index);
int run_add_drive_scratch (ei_x_buff *retbuff, const char *buff, int *index);
int run_add_drive_with_if (ei_x_buff *retbuff, const char *buff, int *index);
int run_add_libvirt_dom (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_clear (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_close (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_defnode (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_defvar (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_get (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_init (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_insert (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_label (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_load (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_ls (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_match (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_mv (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_rm (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_save (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_set (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_setm (ei_x_buff *retbuff, const char *buff, int *index);
int run_aug_transform (ei_x_buff *retbuff, const char *buff, int *index);
int run_available (ei_x_buff *retbuff, const char *buff, int *index);
int run_available_all_groups (ei_x_buff *retbuff, const char *buff, int *index);
int run_base64_in (ei_x_buff *retbuff, const char *buff, int *index);
int run_base64_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_blkdiscard (ei_x_buff *retbuff, const char *buff, int *index);
int run_blkdiscardzeroes (ei_x_buff *retbuff, const char *buff, int *index);
int run_blkid (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_flushbufs (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_getbsz (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_getro (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_getsize64 (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_getss (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_getsz (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_rereadpt (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_setbsz (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_setra (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_setro (ei_x_buff *retbuff, const char *buff, int *index);
int run_blockdev_setrw (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_balance_cancel (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_balance_pause (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_balance_resume (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_balance_status (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_device_add (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_device_delete (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_filesystem_balance (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_filesystem_defragment (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_filesystem_resize (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_filesystem_show (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_filesystem_sync (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_fsck (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_image (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_qgroup_assign (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_qgroup_create (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_qgroup_destroy (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_qgroup_limit (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_qgroup_remove (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_qgroup_show (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_quota_enable (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_quota_rescan (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_replace (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_rescue_chunk_recover (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_rescue_super_recover (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_scrub_cancel (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_scrub_resume (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_scrub_start (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_scrub_status (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_set_seeding (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_subvolume_create (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_subvolume_delete (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_subvolume_get_default (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_subvolume_list (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_subvolume_set_default (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_subvolume_show (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfs_subvolume_snapshot (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfstune_enable_extended_inode_refs (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfstune_enable_skinny_metadata_extent_refs (ei_x_buff *retbuff, const char *buff, int *index);
int run_btrfstune_seeding (ei_x_buff *retbuff, const char *buff, int *index);
int run_c_pointer (ei_x_buff *retbuff, const char *buff, int *index);
int run_canonical_device_name (ei_x_buff *retbuff, const char *buff, int *index);
int run_cap_get_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_cap_set_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_case_sensitive_path (ei_x_buff *retbuff, const char *buff, int *index);
int run_cat (ei_x_buff *retbuff, const char *buff, int *index);
int run_checksum (ei_x_buff *retbuff, const char *buff, int *index);
int run_checksum_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_checksums_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_chmod (ei_x_buff *retbuff, const char *buff, int *index);
int run_chown (ei_x_buff *retbuff, const char *buff, int *index);
int run_clear_backend_setting (ei_x_buff *retbuff, const char *buff, int *index);
int run_command (ei_x_buff *retbuff, const char *buff, int *index);
int run_command_lines (ei_x_buff *retbuff, const char *buff, int *index);
int run_compress_device_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_compress_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_config (ei_x_buff *retbuff, const char *buff, int *index);
int run_copy_attributes (ei_x_buff *retbuff, const char *buff, int *index);
int run_copy_device_to_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_copy_device_to_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_copy_file_to_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_copy_file_to_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_copy_in (ei_x_buff *retbuff, const char *buff, int *index);
int run_copy_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_copy_size (ei_x_buff *retbuff, const char *buff, int *index);
int run_cp (ei_x_buff *retbuff, const char *buff, int *index);
int run_cp_a (ei_x_buff *retbuff, const char *buff, int *index);
int run_cp_r (ei_x_buff *retbuff, const char *buff, int *index);
int run_cpio_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_cryptsetup_close (ei_x_buff *retbuff, const char *buff, int *index);
int run_cryptsetup_open (ei_x_buff *retbuff, const char *buff, int *index);
int run_dd (ei_x_buff *retbuff, const char *buff, int *index);
int run_debug (ei_x_buff *retbuff, const char *buff, int *index);
int run_debug_drives (ei_x_buff *retbuff, const char *buff, int *index);
int run_debug_upload (ei_x_buff *retbuff, const char *buff, int *index);
int run_device_index (ei_x_buff *retbuff, const char *buff, int *index);
int run_df (ei_x_buff *retbuff, const char *buff, int *index);
int run_df_h (ei_x_buff *retbuff, const char *buff, int *index);
int run_disk_create (ei_x_buff *retbuff, const char *buff, int *index);
int run_disk_format (ei_x_buff *retbuff, const char *buff, int *index);
int run_disk_has_backing_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_disk_virtual_size (ei_x_buff *retbuff, const char *buff, int *index);
int run_dmesg (ei_x_buff *retbuff, const char *buff, int *index);
int run_download (ei_x_buff *retbuff, const char *buff, int *index);
int run_download_blocks (ei_x_buff *retbuff, const char *buff, int *index);
int run_download_inode (ei_x_buff *retbuff, const char *buff, int *index);
int run_download_offset (ei_x_buff *retbuff, const char *buff, int *index);
int run_drop_caches (ei_x_buff *retbuff, const char *buff, int *index);
int run_du (ei_x_buff *retbuff, const char *buff, int *index);
int run_e2fsck (ei_x_buff *retbuff, const char *buff, int *index);
int run_e2fsck_f (ei_x_buff *retbuff, const char *buff, int *index);
int run_echo_daemon (ei_x_buff *retbuff, const char *buff, int *index);
int run_egrep (ei_x_buff *retbuff, const char *buff, int *index);
int run_egrepi (ei_x_buff *retbuff, const char *buff, int *index);
int run_equal (ei_x_buff *retbuff, const char *buff, int *index);
int run_exists (ei_x_buff *retbuff, const char *buff, int *index);
int run_extlinux (ei_x_buff *retbuff, const char *buff, int *index);
int run_f2fs_expand (ei_x_buff *retbuff, const char *buff, int *index);
int run_fallocate (ei_x_buff *retbuff, const char *buff, int *index);
int run_fallocate64 (ei_x_buff *retbuff, const char *buff, int *index);
int run_feature_available (ei_x_buff *retbuff, const char *buff, int *index);
int run_fgrep (ei_x_buff *retbuff, const char *buff, int *index);
int run_fgrepi (ei_x_buff *retbuff, const char *buff, int *index);
int run_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_file_architecture (ei_x_buff *retbuff, const char *buff, int *index);
int run_filesize (ei_x_buff *retbuff, const char *buff, int *index);
int run_filesystem_available (ei_x_buff *retbuff, const char *buff, int *index);
int run_filesystem_walk (ei_x_buff *retbuff, const char *buff, int *index);
int run_fill (ei_x_buff *retbuff, const char *buff, int *index);
int run_fill_dir (ei_x_buff *retbuff, const char *buff, int *index);
int run_fill_pattern (ei_x_buff *retbuff, const char *buff, int *index);
int run_find (ei_x_buff *retbuff, const char *buff, int *index);
int run_find0 (ei_x_buff *retbuff, const char *buff, int *index);
int run_find_inode (ei_x_buff *retbuff, const char *buff, int *index);
int run_findfs_label (ei_x_buff *retbuff, const char *buff, int *index);
int run_findfs_uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_fsck (ei_x_buff *retbuff, const char *buff, int *index);
int run_fstrim (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_append (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_attach_method (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_autosync (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_backend (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_backend_setting (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_backend_settings (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_cachedir (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_direct (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_e2attrs (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_e2generation (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_e2label (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_e2uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_hv (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_identifier (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_libvirt_requested_credential_challenge (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_libvirt_requested_credential_defresult (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_libvirt_requested_credential_prompt (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_libvirt_requested_credentials (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_memsize (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_network (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_path (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_pgroup (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_pid (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_program (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_qemu (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_recovery_proc (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_selinux (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_smp (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_sockdir (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_state (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_tmpdir (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_trace (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_umask (ei_x_buff *retbuff, const char *buff, int *index);
int run_get_verbose (ei_x_buff *retbuff, const char *buff, int *index);
int run_getcon (ei_x_buff *retbuff, const char *buff, int *index);
int run_getxattr (ei_x_buff *retbuff, const char *buff, int *index);
int run_getxattrs (ei_x_buff *retbuff, const char *buff, int *index);
int run_glob_expand (ei_x_buff *retbuff, const char *buff, int *index);
int run_grep (ei_x_buff *retbuff, const char *buff, int *index);
int run_grepi (ei_x_buff *retbuff, const char *buff, int *index);
int run_grub_install (ei_x_buff *retbuff, const char *buff, int *index);
int run_head (ei_x_buff *retbuff, const char *buff, int *index);
int run_head_n (ei_x_buff *retbuff, const char *buff, int *index);
int run_hexdump (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_close (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_commit (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_node_add_child (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_node_children (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_node_delete_child (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_node_get_child (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_node_get_value (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_node_name (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_node_parent (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_node_set_value (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_node_values (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_open (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_root (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_value_key (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_value_string (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_value_type (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_value_utf8 (ei_x_buff *retbuff, const char *buff, int *index);
int run_hivex_value_value (ei_x_buff *retbuff, const char *buff, int *index);
int run_initrd_cat (ei_x_buff *retbuff, const char *buff, int *index);
int run_initrd_list (ei_x_buff *retbuff, const char *buff, int *index);
int run_inotify_add_watch (ei_x_buff *retbuff, const char *buff, int *index);
int run_inotify_close (ei_x_buff *retbuff, const char *buff, int *index);
int run_inotify_files (ei_x_buff *retbuff, const char *buff, int *index);
int run_inotify_init (ei_x_buff *retbuff, const char *buff, int *index);
int run_inotify_read (ei_x_buff *retbuff, const char *buff, int *index);
int run_inotify_rm_watch (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_arch (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_distro (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_drive_mappings (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_filesystems (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_format (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_hostname (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_icon (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_major_version (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_minor_version (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_mountpoints (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_osinfo (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_package_format (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_package_management (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_product_name (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_product_variant (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_roots (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_type (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_windows_current_control_set (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_windows_software_hive (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_windows_system_hive (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_get_windows_systemroot (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_is_live (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_is_multipart (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_is_netinst (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_list_applications (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_list_applications2 (ei_x_buff *retbuff, const char *buff, int *index);
int run_inspect_os (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_exit (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_63_optargs (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_close_output (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_only_optargs (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rbool (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rboolerr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rbufferout (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rbufferouterr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rconstoptstring (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rconstoptstringerr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rconststring (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rconststringerr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rhashtable (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rhashtableerr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rint (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rint64 (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rint64err (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rinterr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rstring (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rstringerr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rstringlist (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rstringlisterr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rstruct (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rstructerr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rstructlist (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_rstructlisterr (ei_x_buff *retbuff, const char *buff, int *index);
int run_internal_test_set_output (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_blockdev (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_busy (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_chardev (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_config (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_dir (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_fifo (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_launching (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_lv (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_ready (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_socket (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_symlink (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_whole_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_zero (ei_x_buff *retbuff, const char *buff, int *index);
int run_is_zero_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_isoinfo (ei_x_buff *retbuff, const char *buff, int *index);
int run_isoinfo_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_journal_close (ei_x_buff *retbuff, const char *buff, int *index);
int run_journal_get (ei_x_buff *retbuff, const char *buff, int *index);
int run_journal_get_data_threshold (ei_x_buff *retbuff, const char *buff, int *index);
int run_journal_get_realtime_usec (ei_x_buff *retbuff, const char *buff, int *index);
int run_journal_next (ei_x_buff *retbuff, const char *buff, int *index);
int run_journal_open (ei_x_buff *retbuff, const char *buff, int *index);
int run_journal_set_data_threshold (ei_x_buff *retbuff, const char *buff, int *index);
int run_journal_skip (ei_x_buff *retbuff, const char *buff, int *index);
int run_kill_subprocess (ei_x_buff *retbuff, const char *buff, int *index);
int run_launch (ei_x_buff *retbuff, const char *buff, int *index);
int run_lchown (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_create_all (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_diskgroup_disks (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_diskgroup_name (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_diskgroup_volumes (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_remove_all (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_scan (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_scan_devices (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_volume_hint (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_volume_partitions (ei_x_buff *retbuff, const char *buff, int *index);
int run_ldmtool_volume_type (ei_x_buff *retbuff, const char *buff, int *index);
int run_lgetxattr (ei_x_buff *retbuff, const char *buff, int *index);
int run_lgetxattrs (ei_x_buff *retbuff, const char *buff, int *index);
int run_list_9p (ei_x_buff *retbuff, const char *buff, int *index);
int run_list_devices (ei_x_buff *retbuff, const char *buff, int *index);
int run_list_disk_labels (ei_x_buff *retbuff, const char *buff, int *index);
int run_list_dm_devices (ei_x_buff *retbuff, const char *buff, int *index);
int run_list_filesystems (ei_x_buff *retbuff, const char *buff, int *index);
int run_list_ldm_partitions (ei_x_buff *retbuff, const char *buff, int *index);
int run_list_ldm_volumes (ei_x_buff *retbuff, const char *buff, int *index);
int run_list_md_devices (ei_x_buff *retbuff, const char *buff, int *index);
int run_list_partitions (ei_x_buff *retbuff, const char *buff, int *index);
int run_ll (ei_x_buff *retbuff, const char *buff, int *index);
int run_llz (ei_x_buff *retbuff, const char *buff, int *index);
int run_ln (ei_x_buff *retbuff, const char *buff, int *index);
int run_ln_f (ei_x_buff *retbuff, const char *buff, int *index);
int run_ln_s (ei_x_buff *retbuff, const char *buff, int *index);
int run_ln_sf (ei_x_buff *retbuff, const char *buff, int *index);
int run_lremovexattr (ei_x_buff *retbuff, const char *buff, int *index);
int run_ls (ei_x_buff *retbuff, const char *buff, int *index);
int run_ls0 (ei_x_buff *retbuff, const char *buff, int *index);
int run_lsetxattr (ei_x_buff *retbuff, const char *buff, int *index);
int run_lstat (ei_x_buff *retbuff, const char *buff, int *index);
int run_lstatlist (ei_x_buff *retbuff, const char *buff, int *index);
int run_lstatns (ei_x_buff *retbuff, const char *buff, int *index);
int run_lstatnslist (ei_x_buff *retbuff, const char *buff, int *index);
int run_luks_add_key (ei_x_buff *retbuff, const char *buff, int *index);
int run_luks_close (ei_x_buff *retbuff, const char *buff, int *index);
int run_luks_format (ei_x_buff *retbuff, const char *buff, int *index);
int run_luks_format_cipher (ei_x_buff *retbuff, const char *buff, int *index);
int run_luks_kill_slot (ei_x_buff *retbuff, const char *buff, int *index);
int run_luks_open (ei_x_buff *retbuff, const char *buff, int *index);
int run_luks_open_ro (ei_x_buff *retbuff, const char *buff, int *index);
int run_luks_uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvcreate (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvcreate_free (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvm_canonical_lv_name (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvm_clear_filter (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvm_remove_all (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvm_scan (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvm_set_filter (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvremove (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvrename (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvresize (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvresize_free (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvs (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvs_full (ei_x_buff *retbuff, const char *buff, int *index);
int run_lvuuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_lxattrlist (ei_x_buff *retbuff, const char *buff, int *index);
int run_max_disks (ei_x_buff *retbuff, const char *buff, int *index);
int run_md_create (ei_x_buff *retbuff, const char *buff, int *index);
int run_md_detail (ei_x_buff *retbuff, const char *buff, int *index);
int run_md_stat (ei_x_buff *retbuff, const char *buff, int *index);
int run_md_stop (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkdir (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkdir_mode (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkdir_p (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkdtemp (ei_x_buff *retbuff, const char *buff, int *index);
int run_mke2fs (ei_x_buff *retbuff, const char *buff, int *index);
int run_mke2fs_J (ei_x_buff *retbuff, const char *buff, int *index);
int run_mke2fs_JL (ei_x_buff *retbuff, const char *buff, int *index);
int run_mke2fs_JU (ei_x_buff *retbuff, const char *buff, int *index);
int run_mke2journal (ei_x_buff *retbuff, const char *buff, int *index);
int run_mke2journal_L (ei_x_buff *retbuff, const char *buff, int *index);
int run_mke2journal_U (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkfifo (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkfs (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkfs_b (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkfs_btrfs (ei_x_buff *retbuff, const char *buff, int *index);
int run_mklost_and_found (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkmountpoint (ei_x_buff *retbuff, const char *buff, int *index);
int run_mknod (ei_x_buff *retbuff, const char *buff, int *index);
int run_mknod_b (ei_x_buff *retbuff, const char *buff, int *index);
int run_mknod_c (ei_x_buff *retbuff, const char *buff, int *index);
int run_mksquashfs (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkswap (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkswap_L (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkswap_U (ei_x_buff *retbuff, const char *buff, int *index);
int run_mkswap_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_mktemp (ei_x_buff *retbuff, const char *buff, int *index);
int run_modprobe (ei_x_buff *retbuff, const char *buff, int *index);
int run_mount (ei_x_buff *retbuff, const char *buff, int *index);
int run_mount_9p (ei_x_buff *retbuff, const char *buff, int *index);
int run_mount_local (ei_x_buff *retbuff, const char *buff, int *index);
int run_mount_local_run (ei_x_buff *retbuff, const char *buff, int *index);
int run_mount_loop (ei_x_buff *retbuff, const char *buff, int *index);
int run_mount_options (ei_x_buff *retbuff, const char *buff, int *index);
int run_mount_ro (ei_x_buff *retbuff, const char *buff, int *index);
int run_mount_vfs (ei_x_buff *retbuff, const char *buff, int *index);
int run_mountable_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_mountable_subvolume (ei_x_buff *retbuff, const char *buff, int *index);
int run_mountpoints (ei_x_buff *retbuff, const char *buff, int *index);
int run_mounts (ei_x_buff *retbuff, const char *buff, int *index);
int run_mv (ei_x_buff *retbuff, const char *buff, int *index);
int run_nr_devices (ei_x_buff *retbuff, const char *buff, int *index);
int run_ntfs_3g_probe (ei_x_buff *retbuff, const char *buff, int *index);
int run_ntfscat_i (ei_x_buff *retbuff, const char *buff, int *index);
int run_ntfsclone_in (ei_x_buff *retbuff, const char *buff, int *index);
int run_ntfsclone_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_ntfsfix (ei_x_buff *retbuff, const char *buff, int *index);
int run_ntfsresize (ei_x_buff *retbuff, const char *buff, int *index);
int run_ntfsresize_size (ei_x_buff *retbuff, const char *buff, int *index);
int run_parse_environment (ei_x_buff *retbuff, const char *buff, int *index);
int run_parse_environment_list (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_add (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_del (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_disk (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_expand_gpt (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_get_bootable (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_get_disk_guid (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_get_gpt_attributes (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_get_gpt_guid (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_get_gpt_type (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_get_mbr_id (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_get_mbr_part_type (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_get_name (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_get_parttype (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_init (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_list (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_resize (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_set_bootable (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_set_disk_guid (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_set_disk_guid_random (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_set_gpt_attributes (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_set_gpt_guid (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_set_gpt_type (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_set_mbr_id (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_set_name (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_to_dev (ei_x_buff *retbuff, const char *buff, int *index);
int run_part_to_partnum (ei_x_buff *retbuff, const char *buff, int *index);
int run_ping_daemon (ei_x_buff *retbuff, const char *buff, int *index);
int run_pread (ei_x_buff *retbuff, const char *buff, int *index);
int run_pread_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_pvchange_uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_pvchange_uuid_all (ei_x_buff *retbuff, const char *buff, int *index);
int run_pvcreate (ei_x_buff *retbuff, const char *buff, int *index);
int run_pvremove (ei_x_buff *retbuff, const char *buff, int *index);
int run_pvresize (ei_x_buff *retbuff, const char *buff, int *index);
int run_pvresize_size (ei_x_buff *retbuff, const char *buff, int *index);
int run_pvs (ei_x_buff *retbuff, const char *buff, int *index);
int run_pvs_full (ei_x_buff *retbuff, const char *buff, int *index);
int run_pvuuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_pwrite (ei_x_buff *retbuff, const char *buff, int *index);
int run_pwrite_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_read_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_read_lines (ei_x_buff *retbuff, const char *buff, int *index);
int run_readdir (ei_x_buff *retbuff, const char *buff, int *index);
int run_readlink (ei_x_buff *retbuff, const char *buff, int *index);
int run_readlinklist (ei_x_buff *retbuff, const char *buff, int *index);
int run_realpath (ei_x_buff *retbuff, const char *buff, int *index);
int run_remount (ei_x_buff *retbuff, const char *buff, int *index);
int run_remove_drive (ei_x_buff *retbuff, const char *buff, int *index);
int run_removexattr (ei_x_buff *retbuff, const char *buff, int *index);
int run_rename (ei_x_buff *retbuff, const char *buff, int *index);
int run_resize2fs (ei_x_buff *retbuff, const char *buff, int *index);
int run_resize2fs_M (ei_x_buff *retbuff, const char *buff, int *index);
int run_resize2fs_size (ei_x_buff *retbuff, const char *buff, int *index);
int run_rm (ei_x_buff *retbuff, const char *buff, int *index);
int run_rm_f (ei_x_buff *retbuff, const char *buff, int *index);
int run_rm_rf (ei_x_buff *retbuff, const char *buff, int *index);
int run_rmdir (ei_x_buff *retbuff, const char *buff, int *index);
int run_rmmountpoint (ei_x_buff *retbuff, const char *buff, int *index);
int run_rsync (ei_x_buff *retbuff, const char *buff, int *index);
int run_rsync_in (ei_x_buff *retbuff, const char *buff, int *index);
int run_rsync_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_scrub_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_scrub_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_scrub_freespace (ei_x_buff *retbuff, const char *buff, int *index);
int run_selinux_relabel (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_append (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_attach_method (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_autosync (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_backend (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_backend_setting (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_backend_settings (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_cachedir (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_direct (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_e2attrs (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_e2generation (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_e2label (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_e2uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_hv (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_identifier (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_label (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_libvirt_requested_credential (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_libvirt_supported_credentials (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_memsize (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_network (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_path (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_pgroup (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_program (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_qemu (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_recovery_proc (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_selinux (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_smp (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_tmpdir (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_trace (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_uuid_random (ei_x_buff *retbuff, const char *buff, int *index);
int run_set_verbose (ei_x_buff *retbuff, const char *buff, int *index);
int run_setcon (ei_x_buff *retbuff, const char *buff, int *index);
int run_setxattr (ei_x_buff *retbuff, const char *buff, int *index);
int run_sfdisk (ei_x_buff *retbuff, const char *buff, int *index);
int run_sfdiskM (ei_x_buff *retbuff, const char *buff, int *index);
int run_sfdisk_N (ei_x_buff *retbuff, const char *buff, int *index);
int run_sfdisk_disk_geometry (ei_x_buff *retbuff, const char *buff, int *index);
int run_sfdisk_kernel_geometry (ei_x_buff *retbuff, const char *buff, int *index);
int run_sfdisk_l (ei_x_buff *retbuff, const char *buff, int *index);
int run_sh (ei_x_buff *retbuff, const char *buff, int *index);
int run_sh_lines (ei_x_buff *retbuff, const char *buff, int *index);
int run_shutdown (ei_x_buff *retbuff, const char *buff, int *index);
int run_sleep (ei_x_buff *retbuff, const char *buff, int *index);
int run_stat (ei_x_buff *retbuff, const char *buff, int *index);
int run_statns (ei_x_buff *retbuff, const char *buff, int *index);
int run_statvfs (ei_x_buff *retbuff, const char *buff, int *index);
int run_strings (ei_x_buff *retbuff, const char *buff, int *index);
int run_strings_e (ei_x_buff *retbuff, const char *buff, int *index);
int run_swapoff_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_swapoff_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_swapoff_label (ei_x_buff *retbuff, const char *buff, int *index);
int run_swapoff_uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_swapon_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_swapon_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_swapon_label (ei_x_buff *retbuff, const char *buff, int *index);
int run_swapon_uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_sync (ei_x_buff *retbuff, const char *buff, int *index);
int run_syslinux (ei_x_buff *retbuff, const char *buff, int *index);
int run_tail (ei_x_buff *retbuff, const char *buff, int *index);
int run_tail_n (ei_x_buff *retbuff, const char *buff, int *index);
int run_tar_in (ei_x_buff *retbuff, const char *buff, int *index);
int run_tar_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_tgz_in (ei_x_buff *retbuff, const char *buff, int *index);
int run_tgz_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_touch (ei_x_buff *retbuff, const char *buff, int *index);
int run_truncate (ei_x_buff *retbuff, const char *buff, int *index);
int run_truncate_size (ei_x_buff *retbuff, const char *buff, int *index);
int run_tune2fs (ei_x_buff *retbuff, const char *buff, int *index);
int run_tune2fs_l (ei_x_buff *retbuff, const char *buff, int *index);
int run_txz_in (ei_x_buff *retbuff, const char *buff, int *index);
int run_txz_out (ei_x_buff *retbuff, const char *buff, int *index);
int run_umask (ei_x_buff *retbuff, const char *buff, int *index);
int run_umount (ei_x_buff *retbuff, const char *buff, int *index);
int run_umount_all (ei_x_buff *retbuff, const char *buff, int *index);
int run_umount_local (ei_x_buff *retbuff, const char *buff, int *index);
int run_upload (ei_x_buff *retbuff, const char *buff, int *index);
int run_upload_offset (ei_x_buff *retbuff, const char *buff, int *index);
int run_user_cancel (ei_x_buff *retbuff, const char *buff, int *index);
int run_utimens (ei_x_buff *retbuff, const char *buff, int *index);
int run_utsname (ei_x_buff *retbuff, const char *buff, int *index);
int run_version (ei_x_buff *retbuff, const char *buff, int *index);
int run_vfs_label (ei_x_buff *retbuff, const char *buff, int *index);
int run_vfs_minimum_size (ei_x_buff *retbuff, const char *buff, int *index);
int run_vfs_type (ei_x_buff *retbuff, const char *buff, int *index);
int run_vfs_uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_vg_activate (ei_x_buff *retbuff, const char *buff, int *index);
int run_vg_activate_all (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgchange_uuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgchange_uuid_all (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgcreate (ei_x_buff *retbuff, const char *buff, int *index);
int run_vglvuuids (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgmeta (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgpvuuids (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgremove (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgrename (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgs (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgs_full (ei_x_buff *retbuff, const char *buff, int *index);
int run_vgscan (ei_x_buff *retbuff, const char *buff, int *index);
int run_vguuid (ei_x_buff *retbuff, const char *buff, int *index);
int run_wait_ready (ei_x_buff *retbuff, const char *buff, int *index);
int run_wc_c (ei_x_buff *retbuff, const char *buff, int *index);
int run_wc_l (ei_x_buff *retbuff, const char *buff, int *index);
int run_wc_w (ei_x_buff *retbuff, const char *buff, int *index);
int run_wipefs (ei_x_buff *retbuff, const char *buff, int *index);
int run_write (ei_x_buff *retbuff, const char *buff, int *index);
int run_write_append (ei_x_buff *retbuff, const char *buff, int *index);
int run_write_file (ei_x_buff *retbuff, const char *buff, int *index);
int run_xfs_admin (ei_x_buff *retbuff, const char *buff, int *index);
int run_xfs_growfs (ei_x_buff *retbuff, const char *buff, int *index);
int run_xfs_info (ei_x_buff *retbuff, const char *buff, int *index);
int run_xfs_repair (ei_x_buff *retbuff, const char *buff, int *index);
int run_yara_destroy (ei_x_buff *retbuff, const char *buff, int *index);
int run_yara_load (ei_x_buff *retbuff, const char *buff, int *index);
int run_yara_scan (ei_x_buff *retbuff, const char *buff, int *index);
int run_zegrep (ei_x_buff *retbuff, const char *buff, int *index);
int run_zegrepi (ei_x_buff *retbuff, const char *buff, int *index);
int run_zero (ei_x_buff *retbuff, const char *buff, int *index);
int run_zero_device (ei_x_buff *retbuff, const char *buff, int *index);
int run_zero_free_space (ei_x_buff *retbuff, const char *buff, int *index);
int run_zerofree (ei_x_buff *retbuff, const char *buff, int *index);
int run_zfgrep (ei_x_buff *retbuff, const char *buff, int *index);
int run_zfgrepi (ei_x_buff *retbuff, const char *buff, int *index);
int run_zfile (ei_x_buff *retbuff, const char *buff, int *index);
int run_zgrep (ei_x_buff *retbuff, const char *buff, int *index);
int run_zgrepi (ei_x_buff *retbuff, const char *buff, int *index);

#endif /* GUESTFS_ERLANG_ACTIONS_H_ */
